/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <media_content.h>
#include <app_debug.h>
#include <app_contents.h>
#include <app_media.h>
#include <gridmgr.h>
#include <inputmgr.h>
#include <viewmgr.h>

#include "define.h"
#include "view.h"
#include "data/datamgr.h"
#include "grid/grid.h"
#include "util/util.h"

#define STR_GENRE_NAME "Genre"

static char *_media_text_get(void *data, Evas_Object *obj, const char *part)
{
	app_media *am;
	app_media_info *info;

	if (!data)
		return NULL;

	am = data;

	info = app_media_get_info(am);
	if (!info) {
		_ERR("failed to get media info");
		return NULL;
	}

	if (!strcmp(part, PART_ELM_TEXT_TITLE))
		return strdup(info->title);
	else if (!strcmp(part, PART_ELM_TEXT_ARTIST))
		return strdup(info->audio->artist);
	else if (!strcmp(part, PART_ELM_TEXT_ALBUM))
		return strdup(info->audio->album);

	return NULL;
}

static Evas_Object *_media_content_get(void *data,
			Evas_Object *obj, const char *part)
{
	Evas_Object *eo;
	app_media *am;
	app_media_info *info;

	if (!data)
		return NULL;

	am = data;

	info = app_media_get_info(am);
	if (!info) {
		_ERR("failed to get media info");
		return NULL;
	}

	eo = NULL;
	if (!strcmp(part, PART_ELM_SWALLOW_THUMBNAIL)) {
		eo = util_add_image(obj, info->thumbnail_path);
		if (!eo) {
			_ERR("failed to create image object");
			return NULL;
		}
	} else if (!strcmp(part, PART_ELM_SWALLOW_FAVORITE)) {
		if (!info->favorite)
			return NULL;

		eo = util_add_image(obj, IMAGE_THUMBNAIL_FAVORITE);
		if (!eo) {
			_ERR("failed to create image object");
			return NULL;
		}
	}

	if (eo)
		evas_object_show(eo);

	return eo;
}

static char *_album_text_get(void *data, Evas_Object *obj, const char *part)
{
	struct album_info *ai;

	if (!data)
		return NULL;

	ai = data;

	return strdup(ai->name);
}

static char *_artist_text_get(void *data, Evas_Object *obj, const char *part)
{
	struct album_info *ai;

	if (!data)
		return NULL;

	ai = data;

	return strdup(ai->artist);
}

static Evas_Object *_album_content_get(void *data, Evas_Object *obj,
			const char *part)
{
	Evas_Object *eo;
	struct album_info *ai;
	struct color_data bg;

	if (!data)
		return NULL;

	ai = data;

	eo = NULL;
	if (!strcmp(part, PART_ELM_SWALLOW_THUMBNAIL)) {
		eo = util_add_image(obj, ai->album_art);
		if (!eo) {
			_ERR("failed to create image object");
			return NULL;
		}
	} else if (!strcmp(part, PART_ELM_SWALLOW_TEXTBG)) {
		eo = evas_object_rectangle_add(obj);
		if (!eo) {
			_ERR("failed to create rectangle object");
			return NULL;
		}

		app_contents_get_color(ai->name, NULL, &bg);
		evas_object_color_set(eo, bg.r, bg.g, bg.b, bg.a);
	}

	if (eo)
		evas_object_show(eo);

	return eo;
}

static char *_group_text_get(void *data, Evas_Object *obj, const char *part)
{
	struct group_info *gi;
	char buf[32];
	int cnt;

	if (!data)
		return NULL;

	gi = data;

	if (!strcmp(part, PART_ELM_TEXT_TITLE))
		return strdup(gi->name);
	else if (!strcmp(part, PART_ELM_TEXT_COUNT)) {
		cnt = eina_list_count(gi->list);
		snprintf(buf, sizeof(buf), "%d", cnt);
		return strdup(buf);
	}

	return NULL;
}

static Evas_Object *_group_content_get(void *data, Evas_Object *obj,
			const char *part)
{
	Evas_Object *eo;
	struct group_info *gi;
	struct color_data bg;

	if (!data)
		return NULL;

	gi = data;

	eo = NULL;
	if (!strcmp(part, PART_ELM_SWALLOW_BG)) {
		eo = evas_object_rectangle_add(obj);
		if (!eo) {
			_ERR("failed to create rectangle object");
			return NULL;
		}

		app_contents_get_color(gi->name, &bg, NULL);
		evas_object_color_set(eo, bg.r, bg.g, bg.b, bg.a);

		evas_object_show(eo);
	}

	return eo;
}

static Eina_List *_get_song_list(struct datamgr *dmgr)
{
	if (!dmgr) {
		_ERR("invalid argument");
		return NULL;
	}

	return dmgr->ops->get_group(dmgr->handle, E_MEDIA_NAME, NULL);
}

static Eina_List *_get_album_list(struct datamgr *dmgr)
{
	if (!dmgr) {
		_ERR("invalid argument");
		return NULL;
	}

	return dmgr->ops->get_group(dmgr->handle, E_ALBUM_NAME, NULL);
}

static Eina_List *_get_artist_list(struct datamgr *dmgr)
{
	if (!dmgr) {
		_ERR("invalid argument");
		return NULL;
	}

	return dmgr->ops->get_group(dmgr->handle, E_ALBUM_ARTIST, NULL);
}

static Eina_List *_get_genre_list(struct datamgr *dmgr)
{
	Eina_List *list;
	struct group_info *gi;

	if (!dmgr) {
		_ERR("invalid argument");
		return NULL;
	}

	gi = calloc(1, sizeof(*gi));
	if (!gi) {
		_ERR("failed to allocate memory");
		return NULL;
	}

	gi->name = STR_GENRE_NAME;
	gi->list = dmgr->ops->get_group(dmgr->handle, E_MEDIA_GENRE, NULL);

	list = NULL;
	list = eina_list_append(list, gi);

	return list;
}

static void _free_item_list(struct datamgr *dmgr, Eina_List *list)
{
	if (!dmgr) {
		_ERR("invalid argument");
		return;
	}

	dmgr->ops->free_group(list);
}

static void _free_group_list(struct datamgr *dmgr, Eina_List *list)
{
	struct group_info *gi;

	if (!dmgr) {
		_ERR("invalid argument");
		return;
	}

	EINA_LIST_FREE(list, gi) {
		dmgr->ops->free_group(gi->list);
		free(gi);
	}
}

static void _key_down_cb(void *data, Elm_Object_Item *it,
			Evas_Event_Key_Down *ev)
{
	Eina_List *list;
	struct view_update_data vdata;
	app_media *am;

	if (!data || !ev) {
		_ERR("invalid argument");
		return;
	}

	list = data;

	if (!strcmp(ev->keyname, KEY_MENU) ||
		!strcmp(ev->keyname, KEY_CONTEXT_MENU)) {
		am = elm_object_item_data_get(it);
		if (!am) {
			_ERR("failed to get app media");
			return;
		}

		vdata.list = list;
		vdata.index = util_get_media_index(vdata.list, am);

		viewmgr_update_view(VIEW_ACTION_MENU, UPDATE_CONTENT, &vdata);
		viewmgr_show_view(VIEW_ACTION_MENU);
	}
}

static void _selected_cb(void *data, Elm_Object_Item *it)
{
	Eina_List *list;
	struct view_update_data vdata;
	app_media *am;

	if (!data || !it) {
		_ERR("invalid argument");
		return;
	}

	list = data;

	am = elm_object_item_data_get(it);
	if (!am) {
		_ERR("failed to get app media");
		return;
	}

	vdata.list = list;
	vdata.index = util_get_media_index(vdata.list, am);

	viewmgr_update_view(VIEW_MPLAYER, UPDATE_CONTENT, &vdata);
	viewmgr_push_view(VIEW_MPLAYER);
}

static struct grid_class _gclass[] = {
	[E_MUSIC_SONG] = {
		.item_style = STYLE_MUSIC_SONG,
		.text_get = _media_text_get,
		.content_get = _media_content_get
	},
	[E_MUSIC_ALBUM] = {
		.item_style = STYLE_MUSIC_ALBUM,
		.text_get = _album_text_get,
		.content_get = _album_content_get
	},
	[E_MUSIC_ARTIST] = {
		.item_style = STYLE_MUSIC_ARTIST,
		.text_get = _artist_text_get,
		.content_get = _album_content_get
	},
	[E_MUSIC_GENRE] = {
		.item_style = STYLE_MUSIC_GENRE,
		.text_get = _group_text_get,
		.content_get = _group_content_get
	},
	/* Other view mode class will be added later */
};

static struct grid_data _gdata[] = {
	[E_MUSIC_SONG] = {
		.item_x = MUSIC_SONG_ITEM_X,
		.item_y = MUSIC_SONG_ITEM_Y,
		.item_num = MUSIC_SONG_ITEM_NUM,
		.grid_padding = MUSIC_SONG_GRID_PADDING,
		.gclass = &_gclass[E_MUSIC_SONG],
		.get_item_list = _get_song_list,
		.free_item_list = _free_item_list,
		.key_down_cb = _key_down_cb,
		.selected_cb = _selected_cb
	},
	[E_MUSIC_ALBUM] = {
		.item_x = MUSIC_ALBUM_ITEM_X,
		.item_y = MUSIC_ALBUM_ITEM_Y,
		.item_num = MUSIC_ALBUM_ITEM_NUM,
		.grid_padding = MUSIC_ALBUM_GRID_PADDING,
		.gclass = &_gclass[E_MUSIC_ALBUM],
		.get_item_list = _get_album_list,
		.free_item_list = _free_item_list,
	},
	[E_MUSIC_ARTIST] = {
		.item_x = MUSIC_ARTIST_ITEM_X,
		.item_y = MUSIC_ARTIST_ITEM_Y,
		.item_num = MUSIC_ARTIST_ITEM_NUM,
		.grid_padding = MUSIC_ARTIST_GRID_PADDING,
		.gclass = &_gclass[E_MUSIC_ARTIST],
		.get_item_list = _get_artist_list,
		.free_item_list = _free_item_list,
	},
	[E_MUSIC_GENRE] = {
		.item_x = MUSIC_GENRE_ITEM_X,
		.item_y = MUSIC_GENRE_ITEM_Y,
		.item_num = MUSIC_GENRE_ITEM_NUM,
		.grid_padding = MUSIC_GENRE_GRID_PADDING,
		.gclass = &_gclass[E_MUSIC_GENRE],
		.get_item_list = _get_genre_list,
		.free_item_list = _free_group_list,
	},
	/* Other view mode data will be added later */
};

struct grid_data *get_music_grid_data(int type)
{
	if (type < 0 || type >= E_MUSIC_MAX) {
		_ERR("invalid argument");
		return NULL;
	}

	return &_gdata[type];
}
